@@ -1,13 +1,11 @@
 use anyhow::Result;
 use google_authz::{Credentials, TokenSource};
 use serde::{de::DeserializeOwned, Deserialize, Serialize};
-use std::{collections::HashSet, fs::read_to_string};
-use tiny_firestore_odm::{Collection, Database, NamedDocument};
+use std::collections::HashSet;
+use tiny_firestore_odm::{Collection, CollectionName, Database, NamedDocument};
 use tokio_stream::StreamExt;
 use uuid::Uuid;
 
-const SCOPES: &[&str] = &["https://www.googleapis.com/auth/cloud-platform"];
-
 #[derive(Serialize, Deserialize, PartialEq, Debug, Eq, Hash, Clone)]
 struct User {
     pub name: String,
@@ -16,9 +14,9 @@ struct User {
     pub city: Option<String>,
 }
 
-#[derive(Deserialize)]
-struct ProjectIdExtractor {
-    project_id: String,
+#[derive(Serialize, Deserialize, PartialEq, Debug, Eq, Hash, Clone)]
+struct Device {
+    pub id: String,
 }
 
 async fn empty_collection<T>(collection: &Collection<T>) -> Result<()>
@@ -34,28 +32,32 @@ where
     Ok(())
 }
 
-fn get_source_and_project() -> (TokenSource, String) {
-    let creds_json = if let Ok(creds) = std::env::var("GCP_SERVICE_ACCOUNT") {
-        creds
-    } else {
-        read_to_string("credentials.json")
-            .expect("No credentials.json and GCP_SERVICE_ACCOUNT not set.")
-    };
+async fn get_source_and_project() -> (TokenSource, String) {
+    let project_id = std::env::var("GCP_PROJECT_ID").expect(
+        "The GCP_PROJECT_ID environment variable should point to a Google Cloud project ID.",
+    );
 
-    let source: TokenSource = Credentials::from_json(creds_json.as_bytes(), SCOPES).into();
-    let project: ProjectIdExtractor =
-        serde_json::from_str(&creds_json).expect("Could not parse credentials file");
+    let source: TokenSource = Credentials::default().await.into();
 
-    (source, project.project_id)
+    (source, project_id)
 }
 
 #[tokio::test]
 async fn do_test() {
     let unique_id = Uuid::new_v4().to_string();
 
-    let (token_source, project_id) = get_source_and_project();
+    let (token_source, project_id) = get_source_and_project().await;
     let db = Database::new(token_source, &project_id).await;
-    let users: Collection<User> = db.collection(&format!("tmp-{}", unique_id));
+    let collection_id = format!("tmp-{}", unique_id);
+    let users: Collection<User> = db.collection(&collection_id);
+
+    assert_eq!(
+        CollectionName::parse(&format!(
+            "projects/{}/databases/(default)/documents/{}",
+            project_id, collection_id
+        )).unwrap(),
+        users.name()
+    );
 
     // Create a pair of users.
     let mut u1 = User {
@@ -104,6 +106,10 @@ async fn do_test() {
 
     assert_eq!(u1, u1_updated);
 
+    // Create a subcollection.
+    let devices: Collection<Device> = users.subcollection(u1_key.leaf_name(), "devices");
+    devices.create(&Device {id: "blah".to_string()}).await.unwrap();
+
     // Delete existing documents to create fresh start.
     empty_collection(&users).await.unwrap();
 }
